# authentic2-auth-email - authentic2 authentication for Email
# Copyright (C) 2024 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MEemailHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from random import randint

from django.contrib import messages
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from django.views.generic.edit import FormView

from authentic2.utils.misc import (
    get_next_url,
    is_fully_authenticated,
    make_url,
    redirect,
    remember_authentication,
    send_templated_mail,
)
from authentic2.utils.views import csrf_token_check

from .forms import EnableForm, LoginForm
from .models import EmailAuthenticator


class Profile(TemplateView):
    template_name = 'authentic2_auth_email/profile.html'

    def get_context_data(self, **context):
        try:
            self.request.user.enabled_auth_factors.get(authenticator=EmailAuthenticator.objects.get())
        except self.request.user.enabled_auth_factors.model.DoesNotExist:
            context['enabled'] = False
            if 'auth_level' in self.request.GET:
                # Forced enabling flow
                context['enable_url'] = make_url('email-enable', keep_params=True, request=self.request)
            else:
                context['enable_url'] = make_url(
                    'email-enable', params={'next': self.request.get_full_path()}
                )
        else:
            context['enabled'] = True
        return super().get_context_data(**context)


profile = Profile.as_view()


class Login(FormView):
    template_name = 'authentic2_auth_email/login.html'
    form_class = LoginForm

    def get_context_data(self, **kwargs):
        kwargs['submit_name'] = 'email-submit'
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return get_next_url(self.request.GET) or make_url('account_management')

    def form_valid(self, form):
        csrf_token_check(self.request, form)

        model = EmailAuthenticator.objects.get()
        if self.request.POST.get('email-submit', '') == 'send':
            otp = ''.join([str(randint(0, 9)) for num in range(0, model.code_size)])
            self.request.session['email_otp'] = otp
            context = {'user': self.request.user, 'password': otp}
            send_templated_mail(
                self.request.user.email,
                'authentic2_auth_email/login_email',
                context=context,
            )
            return self.form_invalid(form)

        password = form.cleaned_data['password']
        if password:
            if password == self.request.session.get('email_otp', ''):
                del self.request.session['email_otp']
                remember_authentication(self.request, model)
                return super().form_valid(form)
            else:
                form.add_error('password', _('Invalid password.'))
                return self.form_invalid(form)
        else:
            return self.form_invalid(form)


login = Login.as_view()


class Enable(FormView):
    template_name = 'authentic2_auth_email/enable.html'
    form_class = EnableForm

    def dispatch(self, request, *args, **kwargs):
        if not is_fully_authenticated(request):
            messages.warning(request, 'You must be fully authenticated to activate new factors!')
            return redirect(request, 'auth_login', params={'next': request.path})
        authenticator = EmailAuthenticator.objects.get()
        if not authenticator.enabled:
            messages.warning(request, 'Authenticator not enabled!')
            return redirect(request, 'account_management')

        try:
            request.user.enabled_auth_factors.get(authenticator=authenticator)
            return redirect(request, 'account_management')
        except request.user.enabled_auth_factors.model.DoesNotExist:
            return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return make_url('account_management', keep_params=True, request=self.request)

    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs)

    def form_valid(self, form):
        csrf_token_check(self.request, form)
        self.request.user.enabled_auth_factors.create(authenticator=EmailAuthenticator.objects.get())
        if self.request.user.email:
            send_templated_mail(self.request.user, 'authentic2/factor_enabled')
        return super().form_valid(form)


enable = Enable.as_view()


class Disable(TemplateView):
    template_name = 'authentic2_auth_email/disable_confirm.html'

    def post(self, request):
        if not is_fully_authenticated(request):
            messages.warning(request, 'Fully authenticate to disable a factor!')
            return redirect(request, 'auth_login', param={'next': request.path})
        try:
            factor = request.user.enabled_auth_factors.get(authenticator=EmailAuthenticator.objects.get())
            factor.delete()
        except request.user.enabled_auth_factors.model.DoesNotExist:
            pass
        if request.user.email:
            send_templated_mail(request.user, 'authentic2/factor_disabled')
        return redirect(request, 'account_management')


disable = Disable.as_view()
