# authentic2-auth-rc - authentic2 authentication for RecoveryCodes
# Copyright (C) 2024 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from random import SystemRandom

from django.conf import settings
from django.db import models
from django.utils.translation import gettext as _

from authentic2.apps.authenticators.models import BaseAuthenticator


class RecoveryCodeAuthenticator(BaseAuthenticator):
    _id = 'recovery-codes'
    type = 'recovery-codes'
    how = ['recovery-codes']
    unique = True
    min_required_auth_level = 1

    class Meta:
        verbose_name = _('Recovery codes')

    number_of_codes = models.PositiveSmallIntegerField(
        _('Number of codes to generate'),
        default=10,
        help_text=_('Number of recovery codes to generate for one user'),
    )

    @property
    def manager_form_classes(self):
        from .forms import RecoveryCodeAuthenticatorForm

        return [
            (_('General'), RecoveryCodeAuthenticatorForm),
        ]

    # INFO: 3 magic methods called by `utils_misc.get_authenticator_method`.
    #       They depend on the views… that depend on the model…
    #       That’s why the import are within the methods.
    def login(self, request, *args, **kwargs):
        from .views import login

        return login(request, *args, **kwargs)

    def profile(self, request, *args, **kwargs):
        from .views import profile

        return profile(request, *args, **kwargs)

    def registration(self, request, *args, **kwargs):
        raise NotImplementedError()


class RecoveryCodeManager(models.Manager):
    def generate_for_user(self, user, number=10):
        for _ in range(number):
            code = format(SystemRandom().getrandbits(40), '010x')
            self.create(user=user, code=code)


class RecoveryCode(models.Model):
    objects = RecoveryCodeManager()

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='recovery_codes',
        verbose_name=_('user'),
        on_delete=models.CASCADE,
    )
    code = models.CharField(max_length=10)
