# authentic2-auth-rc - authentic2 authentication for RecoveryCodes
# Copyright (C) 2024 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from django.views.generic.edit import FormView
from django.views.generic.list import ListView

from authentic2.utils.misc import (
    get_next_url,
    is_fully_authenticated,
    make_url,
    redirect,
    remember_authentication,
    send_templated_mail,
)
from authentic2.utils.views import csrf_token_check

from .forms import EnableForm, LoginForm
from .models import RecoveryCode, RecoveryCodeAuthenticator

RECOVERY_DISPLAY = 'allow_recovery_codes_display'


class Profile(TemplateView):
    template_name = 'authentic2_auth_rc/profile.html'

    def get_context_data(self, **context):
        print(2, self.request.user.enabled_auth_factors.all())
        try:
            print(2, 1, RecoveryCodeAuthenticator.objects.get())
            self.request.user.enabled_auth_factors.get(authenticator=RecoveryCodeAuthenticator.objects.get())
            print(3)
        except self.request.user.enabled_auth_factors.model.DoesNotExist:
            print(4)
            context['enabled'] = False
            if 'auth_level' in self.request.GET:
                # Forced enabling flow
                context['enable_url'] = make_url('rc-enable', keep_params=True, request=self.request)
            else:
                context['enable_url'] = make_url('rc-enable', params={'next': self.request.get_full_path()})
        else:
            context['enabled'] = True
        return super().get_context_data(**context)


profile = Profile.as_view()


class Login(FormView):
    template_name = 'authentic2_auth_rc/login.html'
    form_class = LoginForm

    def get_context_data(self, **kwargs):
        kwargs['submit_name'] = 'rc-submit'
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return get_next_url(self.request.GET) or make_url('account_management')

    def form_valid(self, form):
        csrf_token_check(self.request, form)

        code = form.cleaned_data['recovery_code']
        try:
            recovery_code = self.request.user.recovery_codes.get(code=code)
        except self.request.user.recovery_codes.model.DoesNotExist:
            form.add_error('recovery_code', _('Invalid code.'))
            return self.form_invalid(form)

        recovery_code.delete()
        remember_authentication(self.request, RecoveryCodeAuthenticator.objects.get())
        return super().form_valid(form)


login = Login.as_view()


class Enable(FormView):
    template_name = 'authentic2_auth_rc/enable.html'
    form_class = EnableForm

    def dispatch(self, request, *args, **kwargs):
        if not is_fully_authenticated(request):
            messages.warning(request, 'You must be fully authenticated to activate new factors!')
            return redirect(request, 'auth_login', params={'next': request.path})
        authenticator = RecoveryCodeAuthenticator.objects.get()
        if not authenticator.enabled:
            messages.warning(request, 'Authenticator not enabled!')
            return redirect(request, 'account_management')

        try:
            request.user.enabled_auth_factors.get(authenticator=authenticator)
            return redirect(request, 'account_management')
        except request.user.enabled_auth_factors.model.DoesNotExist:
            return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        self.request.session[RECOVERY_DISPLAY] = True
        return make_url('rc-display', keep_params=True, request=self.request)

    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs)

    def form_valid(self, form):
        csrf_token_check(self.request, form)

        self.request.user.enabled_auth_factors.create(authenticator=RecoveryCodeAuthenticator.objects.get())
        model = RecoveryCodeAuthenticator.objects.get()
        RecoveryCode.objects.generate_for_user(self.request.user, model.number_of_codes)
        if self.request.user.email:
            send_templated_mail(self.request.user, 'authentic2/factor_enabled')
        return super().form_valid(form)


enable = Enable.as_view()


class Disable(TemplateView):
    template_name = 'authentic2_auth_rc/disable_confirm.html'

    def post(self, request):
        if not is_fully_authenticated(request):
            messages.warning(request, 'Fully authenticate to disable a factor!')
            return redirect(request, 'auth_login', param={'next': request.path})
        try:
            factor = request.user.enabled_auth_factors.get(
                authenticator=RecoveryCodeAuthenticator.objects.get()
            )
            factor.delete()
        except request.user.enabled_auth_factors.model.DoesNotExist:
            pass
        else:
            request.user.recovery_codes.all().delete()
        if request.user.email:
            send_templated_mail(request.user, 'authentic2/factor_disabled')
        return redirect(request, 'account_management')


disable = Disable.as_view()


class Display(ListView):
    template_name = 'authentic2_auth_rc/display.html'

    def get_context_data(self, **kwargs):
        kwargs['next_url'] = get_next_url(self.request.GET)
        return super().get_context_data(**kwargs)

    def get_queryset(self):
        # Recovery codes should be displayed only once after generation
        if self.request.session.get(RECOVERY_DISPLAY):
            return self.request.user.recovery_codes.all()


display = Display.as_view()


class Generate(TemplateView):
    template_name = 'authentic2_auth_rc/generate_confirm.html'

    def post(self, request):
        request.user.recovery_codes.all().delete()
        model = RecoveryCodeAuthenticator.objects.get()
        RecoveryCode.objects.generate_for_user(request.user, model.number_of_codes)
        request.session[RECOVERY_DISPLAY] = True
        return redirect(request, 'rc-display')


generate = Generate.as_view()
