# authentic2-auth-totp - authentic2 authentication for TOTP
# Copyright (C) 2024 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pyotp
from django.contrib import messages
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from django.views.generic.edit import FormView

from authentic2.utils.misc import (
    get_next_url,
    is_fully_authenticated,
    make_url,
    redirect,
    remember_authentication,
    send_templated_mail,
)
from authentic2.utils.views import csrf_token_check

from .forms import EnableForm, LoginForm
from .models import TOTPAuthenticator, TOTPSecret
from .utils import make_qrcode


class Profile(TemplateView):
    template_name = 'authentic2_auth_totp/profile.html'

    def get_context_data(self, **context):
        try:
            self.request.user.enabled_auth_factors.get(authenticator=TOTPAuthenticator.objects.get())
        except self.request.user.enabled_auth_factors.model.DoesNotExist:
            context['enabled'] = False
            if 'auth_level' in self.request.GET:
                # Forced enabling flow
                context['enable_url'] = make_url('totp-enable', keep_params=True, request=self.request)
            else:
                context['enable_url'] = make_url('totp-enable', params={'next': self.request.get_full_path()})
        else:
            context['enabled'] = True
        return super().get_context_data(**context)


profile = Profile.as_view()


class Login(FormView):
    template_name = 'authentic2_auth_totp/login.html'
    form_class = LoginForm

    def get_context_data(self, **kwargs):
        kwargs['submit_name'] = 'totp-submit'
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return get_next_url(self.request.GET) or make_url('account_management')

    def form_valid(self, form):
        csrf_token_check(self.request, form)

        totp = pyotp.TOTP(self.request.user.totp_secret.key)
        if not totp.verify(form.cleaned_data['one_time_password']):
            form.add_error('one_time_password', _('Invalid password.'))
            return self.form_invalid(form)

        remember_authentication(self.request, TOTPAuthenticator.objects.get())
        return super().form_valid(form)


login = Login.as_view()


class Enable(FormView):
    template_name = 'authentic2_auth_totp/enable.html'
    form_class = EnableForm

    def dispatch(self, request, *args, **kwargs):
        if not is_fully_authenticated(request):
            messages.warning(request, 'You must be fully authenticated to activate new factors!')
            return redirect(request, 'auth_login', params={'next': request.path})
        authenticator = TOTPAuthenticator.objects.get()
        if not authenticator.enabled:
            messages.warning(request, 'Authenticator not enabled!')
            return redirect(request, 'account_management')

        try:
            request.user.enabled_auth_factors.get(authenticator=authenticator)
            return redirect(request, 'account_management')
        except request.user.enabled_auth_factors.model.DoesNotExist:
            return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return make_url('account_management', keep_params=True, request=self.request)

    def get_context_data(self, **kwargs):
        secret = self.get_secret_for_session()
        kwargs['uri'] = make_qrcode(self.request.user, secret)
        encoded_secret = secret.key
        kwargs['secret'] = ' '.join(encoded_secret[i : i + 4] for i in range(0, len(encoded_secret), 4))
        return super().get_context_data(**kwargs)

    def get_secret_for_session(self):
        """Make sure a new temporary secret is generated each session.

        We don't want to generate a secret on every page refresh.
        On the other hand, permanently storing the secret before the user has
        completed setup would allow an attacker who already owns the account to
        preventively get it, so that in case the user decides to enable TOTP
        they would still have access.
        """
        key = self.request.session.setdefault('totp_secret', pyotp.random_base32())
        return TOTPSecret(self.request.user.id, key)

    def form_valid(self, form):
        csrf_token_check(self.request, form)

        secret = self.get_secret_for_session()
        totp = pyotp.TOTP(secret.key)
        if totp.verify(form.cleaned_data['one_time_password']):
            secret.save()
            self.request.user.enabled_auth_factors.create(authenticator=TOTPAuthenticator.objects.get())
            del self.request.session['totp_secret']
            if self.request.user.email:
                send_templated_mail(self.request.user, 'authentic2/factor_enabled')
            return super().form_valid(form)

        form.add_error('one_time_password', _('Invalid password.'))
        return self.form_invalid(form)


enable = Enable.as_view()


class Disable(TemplateView):
    template_name = 'authentic2_auth_totp/disable_confirm.html'

    def post(self, request):
        if not is_fully_authenticated(request):
            messages.warning(request, 'Fully authenticate to disable a factor!')
            return redirect(request, 'auth_login', param={'next': request.path})
        try:
            factor = request.user.enabled_auth_factors.get(authenticator=TOTPAuthenticator.objects.get())
            factor.delete()
        except request.user.enabled_auth_factors.model.DoesNotExist:
            pass
        else:
            request.user.totp_secret.delete()
        if request.user.email:
            send_templated_mail(request.user, 'authentic2/factor_disabled')
        return redirect(request, 'account_management')


disable = Disable.as_view()
